using System;
using Unit4.CollectionsLib;
using Unit4.UtilsLib;
using Unit4.TurtleLib;
using System.Text;

namespace ConsoleApplicationPolygon
{
    class Polygon
    {
        private int nVerts;
        private List<Point> verts;
        public Polygon()
        {
            this.nVerts = 0;
            this.verts = new List<Point>();
        }
        public Polygon(int nVerts, Point center, double rad)
        {
            this.verts = new List<Point>();
            Node<Point> pos = this.verts.GetFirst();
            double x, y, angle;
            this.nVerts = nVerts;
            angle = 2*Math.PI / nVerts;
            for (int i = 0; i < nVerts; i++)
            {
                x = center.GetX() + rad * Math.Cos(angle * i);
                y = center.GetY() + rad * Math.Sin(angle * i);
                Point p = new Point(x, y);
                pos = this.verts.Insert(pos, p);
            }
        }
        public void AddPoint(Point p)
        {
            Node<Point> prev = null;
            Node<Point> pos = this.verts.GetFirst();
            while (pos != null)
            {
                prev = pos;
                pos = pos.GetNext();
            }
            this.verts.Insert(prev, p);
            this.nVerts++;
        }
        public List<Point> GetAllPoints()
        {
            return this.verts;
        }
        public bool Exists(Point p)
        {
            Node<Point> pos = this.verts.GetFirst();
            Point curP;
            while (pos != null)
            {
                curP = pos.GetInfo();
                if (curP.IsEquals(p))
                    return true;
                pos = pos.GetNext();
            }
            return false;
        }
        public void Move(double dx, double dy)
        {
            Node<Point> pos = this.verts.GetFirst();
            Point curP;
            while (pos != null)
            {
                curP = pos.GetInfo();
                curP.SetX(curP.GetX() + dx);
                curP.SetY(curP.GetY() + dy);
                pos = pos.GetNext();
            }
        }
        public override string ToString()
        {
            return "Vertices of Polygon:" + this.verts.ToString();
        }
        public void Draw()
        {
            Node<Point> pos = this.verts.GetFirst();
            Point prevP = new Point(0, 0);
            Point nextP; ;
            double dx, dy;
            double oldHeading = 0;
            double newHeading;
            double heading;
            double steps;
            if (pos == null)
                return;
            Turtle t = new Turtle();
            // t.SetVisible(false);
            // t.SetDelay(0);
            // Move From Origin to first point 
            nextP = pos.GetInfo();
            dx = nextP.GetX() - prevP.GetX();
            dy = nextP.GetY() - prevP.GetY();
            newHeading = GetHeading(dx, dy);
            heading = newHeading - oldHeading;
            oldHeading = newHeading;
            // Convert From Radians To degrees
            heading = heading * 90/ Math.PI;
            steps = prevP.Distance(nextP);
            t.TurnRight(heading);
            //Move the turtle forward steps times along its current heading.
            t.MoveForward(steps);
            t.TailDown();
            prevP = nextP;
            pos = pos.GetNext();
            while (pos != null)
            {
                nextP = pos.GetInfo();
                dx = nextP.GetX() - prevP.GetX();
                dy = nextP.GetY() - prevP.GetY();
                newHeading = GetHeading(dx, dy);
                heading = newHeading - oldHeading;
                oldHeading = newHeading;
                // Convert From Radians To degrees
                heading = heading * 180 / Math.PI;
                steps = prevP.Distance(nextP);
                t.TurnRight(heading);
                //Move the turtle forward steps times along its current heading.
                t.MoveForward(steps);
                prevP = nextP;
                pos = pos.GetNext();
            }
            // Move to First Point
            nextP = this.verts.GetFirst().GetInfo();
            dx = nextP.GetX() - prevP.GetX();
            dy = nextP.GetY() - prevP.GetY();
            newHeading = GetHeading(dx, dy);
            heading = newHeading - oldHeading;
            oldHeading = newHeading;
            // Convert From Radians To degrees
            heading = heading * 180 / Math.PI;
            steps = prevP.Distance(nextP);
            t.TurnRight(heading);
            //Move the turtle forward steps times along its current heading.
            t.MoveForward(steps);


        }
        private static double GetHeading(double dx, double dy)
        {
            double alfa;
            if (dy == 0)
            {
                if (dx > 0)
                    return Math.PI / 2;
                else
                    if (dx < 0)
                        return -Math.PI / 2;
                    return 0;
            }
            alfa = Math.Atan(dx / dy);
            if (dy < 0)
                alfa = alfa + Math.PI;
            return alfa;

        }

    }
}
